#-----------------------------------------------------------------------------
# Unbend a MT.
# Sep 10, 2006
# Copyright (c) 2006 by David G. Drumright (ockham)
# You start with the figure and the MT you already produced for the
# bent joint.  Have the joint bent in the position you want.
#-----------------------------------------------------------------------------

from Tkinter import *
import poser
scene = poser.Scene()
XR=poser.kParmCodeXROT
YR=poser.kParmCodeYROT
ZR=poser.kParmCodeZROT

global F,A
F=scene.CurrentFigure()
A=None
A=scene.CurrentActor()

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global F,A

        self.master = master
        master.title("Unbend Morph")

        self.StatusEntry = Entry(self.master,width=60)
        self.StatusEntry.grid(row=0,column=0,columnspan=3)
        if not F:
            self.StatusEntry.insert(0,'No figure selected! Close and try again.')
        else:
            self.StatusEntry.insert(0,'Fig is %s, actor is %s.  Find MT, give angle.' % (F.Name(),A.Name()) )

        # Lists
        self.ListScroll = Scrollbar(self.master, orient=VERTICAL)
        self.ListScroll.grid( row=2, column=0,sticky=N+S+E)
        self.List = Listbox(self.master, height=15, width=40, selectmode=SINGLE,yscrollcommand = self.ListScroll.set,exportselection=0)
        self.List.grid( row=2, column=1)
        # Set the scrollbar to work
        self.ListScroll["command"] = self.List.yview
        # Fill the list
        for P in A.Parameters():
            if P.IsMorphTarget():
                self.List.insert(END,P.Name())

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents()
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        # Get selection.
        sels=self.List.curselection()
        MTname=self.List.get( int(sels[0]) )
        P=A.Parameter(MTname)

        # Take the selected part.  Find the nonzero MT values:
        MTvalid={}
        G = A.Geometry()
        for i in range(G.NumVertices()):
            Vq=None
            try:
                Vq=P.MorphTargetDelta(i)
            except:
                pass
            if Vq and (Vq != 0.0):
                MTvalid[i]=Vq

        # Now look at those verts only, and get a diff for each.
        Pos0={}
        Pos1={}
        Diff={}

        # Assume the current position of joint is the desired end pos.
        Px=A.ParameterByCode(XR)
        Py=A.ParameterByCode(YR)
        Pz=A.ParameterByCode(ZR)
        X1=Px.Value()
        Y1=Py.Value()
        Z1=Pz.Value()

        # Get end pos values of the verts.
        scene.DrawAll()
        for i in MTvalid.keys():
            WV=G.WorldVertex(i)
            Pos1[i]=( (WV.X(),WV.Y(),WV.Z()) )

        # Now force zero
        Px.SetValue(0.0)
        Py.SetValue(0.0)
        Pz.SetValue(0.0)
        # Get zero pos values of the verts
        scene.DrawAll()
        for i in MTvalid.keys():
            WV=G.WorldVertex(i)
            Pos0[i]=( (WV.X(),WV.Y(),WV.Z()) )

        # Now apply the diffs to the MTs.
        for i in MTvalid.keys():
            (Xm,Ym,Zm)=MTvalid[i]
            (X0,Y0,Z0)=Pos0[i]
            (X1,Y1,Z1)=Pos1[i]
            Xnew = Xm - (X1-X0)
            Ynew = Ym - (Y1-Y0)
            Znew = Zm - (Z1-Z0)
            P.SetMorphTargetDelta(i,Xnew,Ynew,Znew)


        self.ShowStatus('Done modifying %s' % MTname)

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
